@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007-2010 Herve Quatremain
+   Copyright (C) 2007-2013 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -62,6 +62,7 @@ extern int errno;
 #include <net_module.h>
 #include <net_utils_init.h>
 #include <lib_functions.h>
+#include <control_groups_linux.h>
 #include <job_launcher.h>
 
 
@@ -345,6 +346,11 @@ launcher (job_run_t *j, int status_pipe_father[2], const char *job_id)
 	}
 	fcntl (3, F_SETFD, FD_CLOEXEC);
 
+#if HAVE_LIBCGROUP
+	/* Linux Control Group */
+	cgroup_set (j->cgroup_name, getpid());
+#endif
+
 	/* umask */
 #if HAVE_UMASK
 	umask (j->mask);
@@ -703,6 +709,12 @@ submit_job (	job_parameters_t *j, const char *job_id, int *job_pid,
 			return -2;
 	}
 
+	if (add_cgroup_job_run (&jr, j->cgroup_name) != 0) {
+		destroy_job_run (&jr);
+		build_errmsg (NULL, _("Memory allocation error"), err_msg);
+		return -1;
+	}
+
 	switch (add_command_job_run (&jr, j->command, &(j->arguments),
 					j->load_env, &idx))
 	{
