@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -21,10 +21,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -74,7 +70,7 @@ extern int errno;
 #include <lib_functions.h>
 #include <job_launcher_main.h>
 
-/* 
+/*
  * Big number for the alarm() system call used to compute the job duration.
  * On some system (GNU/linux on ppc for instance) this number must not be
  * to big due to limits.  The value here should be fine
@@ -130,7 +126,7 @@ launcher_main (pid_t child, const char *job_id)
 	long int facility;
 	char *file_name;
 	int status, ret;
-	size_t l;
+	ssize_t l;
 	unsigned int remaining;
 	pid_t c;
 	struct sigaction sa;
