@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2013 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -26,6 +26,8 @@
  *        command --> The command to be run
  *    file_stdout --> File name of the command stdout redirection
  *    file_stderr --> File name of the command stderr redirection
+ *    cgroup_name --> Linux Control Group name (path from a controller mount
+ *                    directory)
  *           work --> Working directory (a chdir(2) will be done in this
  *                    directory before running (exec) the user command)
  *          shell --> User default shell
@@ -102,6 +104,7 @@ init_job_run (job_run_t *j)
 		j->command = NULL;
 		j->file_stderr = NULL;
 		j->file_stdout = NULL;
+		j->cgroup_name = NULL;
 		j->work = NULL;
 		j->shell = NULL;
 		j->uid = SCHEDWI_UID_NOBODY;
@@ -131,6 +134,9 @@ destroy_job_run (job_run_t *j)
 		if (j->file_stdout != NULL) {
 			free (j->file_stdout);
 		}
+		if (j->cgroup_name != NULL) {
+			free (j->cgroup_name);
+		}
 		if (j->work != NULL) {
 			free (j->work);
 		}
@@ -801,6 +807,41 @@ add_outfile_job_run (	job_run_t *j, const char *file_stdout,
 }
 
 
+/*
+ * Store the provided Linux Control Group name in the structure.
+ *
+ * Return:
+ *    0 --> No error
+ *   -1 --> Memory allocation error
+ */
+int
+add_cgroup_job_run (job_run_t *j, const char *cgroup_name)
+{
+	char *s;
+
+	if (j == NULL) {
+		return 0;
+	}
+	if (cgroup_name == NULL || cgroup_name[0] == '\0') {
+		if (j->cgroup_name != NULL) {
+			free (j->cgroup_name);
+			j->cgroup_name = NULL;
+		}
+		return 0;
+	}
+	s = (char *) malloc (schedwi_strlen (cgroup_name) + 1);
+	if (s == NULL) {
+		return -1;
+	}
+	strcpy (s, cgroup_name);
+	if (j->cgroup_name != NULL) {
+		free (j->cgroup_name);
+	}
+	j->cgroup_name = s;
+	return 0;
+}
+
+
 /*
  * Compose the program string to run and the arguments.
  * If `load_env' is 0, the provided command will be directly executed.
