@@ -1,6 +1,6 @@
 /* Schedwi
    Copyright (C) 2007 Herve Quatremain
-     
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -33,10 +33,10 @@
 int
 module_load_clnt ()
 {
-	if (	   add_module ("runjob-0.1", run_job) != 0
-		|| add_module ("killjob-0.1", kill_jobs) != 0
-		|| add_module ("testfile-0.1", test_file) != 0
-		|| add_module ("joblist-0.1", get_joblist) != 0
+	if (	   add_module ("runjob", run_job) != 0
+		|| add_module ("killjob", kill_jobs) != 0
+		|| add_module ("testfile", test_file) != 0
+		|| add_module ("joblist", get_joblist) != 0
 		|| add_module ("version", get_version) != 0)
 	{
 		destroy_modules ();
