@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007-2010 Herve Quatremain
+   Copyright (C) 2007-2013 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -56,6 +56,7 @@ enum steps {
 	S_KEY_USER,
 	S_KEY_STDOUT,
 	S_KEY_STDERR,
+	S_KEY_CGROUP,
 	S_KEY_ARGS,
 	S_ARG_ARRAY_START,
 	S_KEY_USERENV,
@@ -153,6 +154,16 @@ json_char_callback (void* ctx, int type, const JSON_value* value)
 				}
 				s->step = S_DATA;
 			}
+			else if (s->step == S_KEY_CGROUP) {
+				if (add_cgroup_job_parameters (&(s->job),
+						value->vu.str.value,
+						value->vu.str.length) != 0)
+				{
+					s->error = 1;
+					return 1;
+				}
+				s->step = S_DATA;
+			}
 			else if (s->step == S_ARG_ARRAY_START) {
 				if (add_arg_job_parameters (&(s->job),
 						value->vu.str.value,
@@ -226,6 +237,12 @@ json_char_callback (void* ctx, int type, const JSON_value* value)
 					s->step = S_KEY_STDERR;
 				}
 				else
+				if (schedwi_strcasecmp (value->vu.str.value,
+							"cgroup") == 0)
+				{
+					s->step = S_KEY_CGROUP;
+				}
+				else
 				if (schedwi_strcasecmp (value->vu.str.value,
 							"user env") == 0)
 				{
