@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2010 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -52,10 +52,131 @@ extern int errno;
 #include <assert.h>
 #endif
 
-#include <lwc_linkedlist.h>
 #include <lib_functions.h>
 #include <utils.h>
 #include <result_mgnt.h>
+#include <JSON_parser.h>
+
+enum steps {
+	S_START,
+	S_OUTER_ARRAY,
+	S_OUTER_ARRAY_MODULE,
+	S_DATA,
+	S_KEY_JOBID,
+	S_KEY_DURATION,
+	S_KEY_KILLED,
+	S_KEY_RETURN_CODE,
+	S_END
+};
+
+struct json_cb {
+	enum steps step;
+	char error;
+	result_t *ptr;
+};
+
+
+/*
+ * JSON callback function
+ */
+static int
+json_char_callback (void* ctx, int type, const JSON_value* value)
+{
+	struct json_cb *s = (struct json_cb *)ctx;
+	char *v;
+
+	switch (type) {
+		case JSON_T_ARRAY_BEGIN:
+			if (s->step == S_START) {
+				s->step = S_OUTER_ARRAY;
+			}
+			break;
+		case JSON_T_STRING:
+			if (s->step == S_OUTER_ARRAY) {
+				s->step = S_OUTER_ARRAY_MODULE;
+			}
+			else if (s->step == S_KEY_JOBID) {
+				v = (char *)malloc (value->vu.str.length + 1);
+				if (v == NULL) {
+					s->error = 1;
+					return 1;
+				}
+				strncpy (v, value->vu.str.value,
+						value->vu.str.length);
+				v[value->vu.str.length] = '\0';
+				if (s->ptr->job_id != NULL) {
+					free (s->ptr->job_id);
+				}
+				s->ptr->job_id     = v;
+				s->ptr->job_id_len = value->vu.str.length;
+				s->step = S_DATA;
+			}
+			break;
+		case JSON_T_OBJECT_BEGIN:
+			if (s->step == S_OUTER_ARRAY_MODULE) {
+				s->step = S_DATA;
+			}
+			break;
+		case JSON_T_KEY:
+			if (s->step == S_DATA) {
+				if (schedwi_strcasecmp (value->vu.str.value,
+							"job id") == 0)
+				{
+					s->step = S_KEY_JOBID;
+				}
+				else
+				if (schedwi_strcasecmp (value->vu.str.value,
+							"duration") == 0)
+				{
+					s->step = S_KEY_DURATION;
+				}
+				else
+				if (schedwi_strcasecmp (value->vu.str.value,
+							"killed") == 0)
+				{
+					s->step = S_KEY_KILLED;
+				}
+				else
+				if (schedwi_strcasecmp (value->vu.str.value,
+							"return code") == 0)
+				{
+					s->step = S_KEY_RETURN_CODE;
+				}
+			}
+			break;
+		case JSON_T_INTEGER:
+			if (s->step == S_KEY_DURATION) {
+				s->ptr->duration = (time_t)
+						value->vu.integer_value;
+				s->step = S_DATA;
+			}
+			else
+			if (s->step == S_KEY_RETURN_CODE) {
+				s->ptr->exit_code = value->vu.integer_value;
+				s->step = S_DATA;
+			}
+			break;
+		case JSON_T_TRUE:
+			if (s->step == S_KEY_KILLED) {
+				s->ptr->was_killed = 1;
+				s->step = S_DATA;
+			}
+			break;
+		case JSON_T_FALSE:
+			if (s->step == S_KEY_KILLED) {
+				s->ptr->was_killed = 0;
+				s->step = S_DATA;
+			}
+			break;
+		case JSON_T_OBJECT_END:
+			if (s->step == S_DATA) {
+				s->step = S_END;
+			}
+			break;
+	}
+
+	return 1;
+}
 
 
 /*
@@ -250,7 +371,7 @@ result_to_file (const result_t *r, const char *job_id, const char *file_name)
 		return -1;
 	}
 
-	return 0; 
+	return 0;
 }
 
 
@@ -299,7 +420,7 @@ result_from_file (result_t *r, const char *file_name)
 
 
 /*
- * Convert a result_t structure to a string
+ * Convert a result_t structure to a string ready to be sent to the server
  *
  * Return:
  *   0 --> No error (buffer contains the string. It is allocated if NULL or
@@ -312,13 +433,21 @@ result_to_str (result_t *r, char **buffer, size_t *buffer_len)
 {
 	size_t len;
 	int ret;
+	const char *s = "[ \"result\", \
+{ \
+ \"job id\"     : \"%s\", \
+ \"duration\"   : %ld, \
+ \"killed\"     : %s, \
+ \"return code\": %d \
+} ]";
+
 
 #if HAVE_ASSERT_H
 	assert (   r != NULL && r->job_id != NULL
 		&& buffer != NULL && buffer_len != NULL);
 #endif
 
-	len = 5 * 4 + r->job_id_len + 55; 
+	len = schedwi_strlen (s) + r->job_id_len + 50;
 	if (len > *buffer_len || *buffer == NULL) {
 		if (*buffer != NULL) {
 			free (*buffer);
@@ -330,13 +459,12 @@ result_to_str (result_t *r, char **buffer, size_t *buffer_len)
 		*buffer_len = len;
 	}
 
-	ret = snprintf (*buffer, *buffer_len,
-			"JID:%s\nDUR:%ld\nKIL:%c\nCOD:%d\n",
+	ret = snprintf (*buffer, *buffer_len, s,
 			r->job_id,
 			r->duration,
-			(r->was_killed == 0) ? '0': '1',
+			(r->was_killed == 0) ? "false": "true",
 			r->exit_code);
-	if (ret >= *buffer_len || ret < 0) {
+	if (ret < 0 || (size_t)ret >= *buffer_len) {
 		return -3;
 	}
 	return 0;
@@ -344,104 +472,58 @@ result_to_str (result_t *r, char **buffer, size_t *buffer_len)
 
 
 /*
- * Convert the string to a list of result_t structure
+ * Convert the string to a result_t structure.
  *
  * Return:
- *   0 --> No error (list is set and must be freed by the caller)
+ *   0 --> No error (result is set and must be freed by the caller by
+ *         destroy_result())
  *  -1 --> Memory allocation error
  */
 int
-str_to_results (const char *buffer, size_t buffer_size, lwc_LL **list)
+str_to_result (const char *buffer, int buffer_size, result_t **result)
 {
-	lwc_LL *tmp_list;
-	char *buff_cpy, *str;
-	result_t *ptr;
+	int i;
+        JSON_config config;
+        struct JSON_parser_struct *jc;
+	struct json_cb cb_data;
+
 
 #if HAVE_ASSERT_H
-	assert (list != NULL);
+	assert (buffer != NULL && result != NULL);
 #endif
 
-	tmp_list = lwc_newLL ();
-	if (tmp_list == NULL) {
+	/* JSON parser initialization */
+	init_JSON_config (&config);
+
+	cb_data.ptr                   = new_result ();
+	if (cb_data.ptr == NULL) {
 		return -1;
 	}
+	cb_data.step                  = S_START;
+	cb_data.error                 = 0;
+	config.depth                  = 20;
+	config.callback               = &json_char_callback;
+	config.allow_comments         = 0;
+	config.handle_floats_manually = 0;
+	config.callback_ctx           = &cb_data;
 
-	if (buffer == NULL || buffer_size <= 0) {
-		*list = tmp_list;
-		return 0;
-	}
+	jc = new_JSON_parser (&config);
 
-	/* Copy the input buffer to add a '\0' at the end (for strtok below) */
-	buff_cpy = (char *) malloc (buffer_size + 1);
-	if (buff_cpy == NULL) {
-		lwc_delLL (tmp_list, NULL);
-		return -1;
-	}
-	strncpy (buff_cpy, buffer, buffer_size);
-	buff_cpy [buffer_size] = '\0';
-
-	/* Parse each line */
-	ptr = NULL;
-	for (	str = strtok (buff_cpy, "\n");
-		str != NULL;
-		str = strtok (NULL, "\n"))
-	{
-		/* Skip the spaces at the begining of the line */
-		while (*str != '\0' && isspace (*str) != 0) {
-			str++;
-		}
-
-		/* Job ID */
-		if (schedwi_strncasecmp (str, "JID:", 4) == 0) {
-			/* A new result */
-			ptr = new_result ();
-			if (ptr == NULL || lwc_addEndLL (tmp_list, ptr) != 0) {
-				destroy_result (ptr);
-				lwc_delLL (tmp_list,
-					(void (*)(const void *))destroy_result);
-				free (buff_cpy);
-				return -1;
-			}
-			ptr->job_id_len = schedwi_strlen (str + 4);
-			ptr->job_id = (char *) malloc (ptr->job_id_len + 1);
-			if (ptr->job_id == NULL) {
-				lwc_delLL (tmp_list,
-					(void (*)(const void *))destroy_result);
-				free (buff_cpy);
-				return -1;
-			}
-			strcpy (ptr->job_id, str + 4);
-			continue;
-		}
-
-		/* Duration */
-		if (schedwi_strncasecmp (str, "DUR:", 4) == 0) {
-			if (ptr != NULL) {
-				ptr->duration = (time_t)
-						strtol (str + 4, NULL, 0);
-			}
-			continue;
-		}
-
-		/* Has the job been killed? */
-		if (schedwi_strncasecmp (str, "KIL:", 4) == 0) {
-			if (ptr != NULL) {
-				ptr->was_killed = (str[4] == '0') ? 0: 1;
-			}
-			continue;
-		}
-
-		/* Return code or signal */
-		if (schedwi_strncasecmp (str, "COD:", 4) == 0) {
-			if (ptr != NULL) {
-				ptr->exit_code = atoi (str + 4);
-			}
-			continue;
+	/* Parse the JSON string */
+	for (i = 0; i < buffer_size && cb_data.step != S_END; i++) {
+		/*
+		 * No need to check the return code as the JSON string has
+		 * already been parsed once in net_parse.c
+		 */
+		JSON_parser_char (jc, buffer[i]);
+		if (cb_data.error != 0) {
+			delete_JSON_parser (jc);
+			destroy_result (cb_data.ptr);
+			return -1;
 		}
 	}
-
-	free (buff_cpy);
-	*list = tmp_list;
+	delete_JSON_parser (jc);
+	*result = cb_data.ptr;
 	return 0;
 }
 
