@@ -1,6 +1,6 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
-     
+   Copyright (C) 2007-2011 Herve Quatremain
+
    This file is part of Schedwi.
 
    Schedwi is free software; you can redistribute it and/or modify
@@ -21,10 +21,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -76,6 +72,43 @@ extern char **environ;
 #endif
 
 
+/*
+ * Convert a unsigned long long int to a string
+ *
+ * Return:
+ *   The number of characters copied in str
+ */
+unsigned int
+copy_ulltostr (unsigned long long int v, char *str)
+{
+	char c;
+	unsigned int len, i, j;
+
+	if (str == NULL) {
+	    return 0;
+	}
+
+	i = 0;
+	do {
+		str[i++] = '0' + v % 10;
+		v /= 10;
+	} while (v > 0);
+	str[i] = '\0';
+	len = i;
+
+	/* Reverse the string */
+	j = 0;
+	i--;
+	while (j < i) {
+	    c = str[i];
+	    str[i--] = str[j];
+	    str[j++] = c;
+	}
+
+	return len;
+}
+
+
 /*
  * Return the filename part of a path
  *
@@ -112,9 +145,9 @@ void
 version (const char *prog_name)
 {
 	printf ("%s (Schedwi) %s\n", prog_name, PACKAGE_VERSION);
-	puts("Copyright (C) 2007 Herve Quatremain");
+	puts("Copyright (C) 2007-2010 Herve Quatremain");
 	puts("Schedwi is free software, covered by the GNU");
-	puts("General Public License version 2 or later.");
+	puts("General Public License version 3 or later.");
 	puts("You may redistribute copies of Schedwi under");
 	puts("the terms of this license.");
 }
@@ -225,6 +258,7 @@ sanitise (int fd)
 void
 clean_process ()
 {
+	int unused;
 #if HAVE_GETENV && HAVE_CHDIR
 	char *tmp;
 #endif
@@ -242,9 +276,9 @@ clean_process ()
 #endif /* HAVE_GETENV */
 	if (tmp == NULL) {
 #ifdef P_tmpdir
-		chdir (P_tmpdir);
+		unused = chdir (P_tmpdir);
 #else
-		chdir ("/tmp");
+		unused = chdir ("/tmp");
 #endif
 	}
 #endif /* HAVE_CHDIR */
@@ -286,14 +320,6 @@ go_daemon()
 	setsid();
 #endif
 
-	p = fork ();
-	if (p < 0) {
-		return -1;
-	}
-	if (p > 0) {
-		exit (0);
-	}
-
 	clean_process ();
 	return 0;
 }
@@ -314,6 +340,7 @@ write_pid_file (const char *file_name)
 {
 	char pid_str[26];
 	size_t l;
+	ssize_t unused;
 
 	if (file_name == NULL) {
 		return 0;
@@ -342,7 +369,7 @@ write_pid_file (const char *file_name)
 	/* Write the PID to the file */
 	l = snprintf (pid_str, 26, "%ld\n", (long int)getpid ());
 	pid_str [26 - 1] = '\0';
-	write (fd_pid_file, pid_str, schedwi_strlen (pid_str));
+	unused = write (fd_pid_file, pid_str, schedwi_strlen (pid_str));
 	return 0;
 }
 
