@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -213,7 +213,7 @@ start_job (const char *path_to_cmd, const char *user_name)
 {
 	int sock;
 	struct sockaddr_in sad;
-	char buff[4096];
+	char buff[4096], *s;
 	char result;
 	ssize_t nb_read;
 	struct hostent *he;
@@ -245,49 +245,48 @@ start_job (const char *path_to_cmd, const char *user_name)
 		return 10;
 	}
 
-	/* Build and send the header */
-	schedwi_memset (buff, 0, 25);
-	buff[0] = 'r';
-	buff[1] = 'u';
-	buff[2] = 'n';
-	buff[3] = 'j';
-	buff[4] = 'o';
-	buff[5] = 'b';
-	buff[6] = '-';
-	buff[7] = '0';
-	buff[8] = '.';
-	buff[9] = '1';
-	send (sock, buff, 25, 0);
-
 	/* Write the request */
-	send (sock, "JID:20070324_23\n", 16, 0);
+	s = "[ \"runjob\", \
+               { \
+		\"job id\"  : \"20070324_23\",\
+		\"args\"    : [ \"arg1\", \"arg2\" ],\
+		\"user env\": false,\n";
+	send (sock, s, schedwi_strlen (s), 0);
 
-	send (sock, "CMD:", 4, 0);
+	s = "\"command\" : \"";
+	send (sock, s, schedwi_strlen (s), 0);
 	send (sock, path_to_cmd, schedwi_strlen (path_to_cmd), 0);
 	send (sock, "/", 1, 0);
-	send (sock, "${CMD}\n", 7, 0);
+	send (sock, "${CMD}", 6, 0);
+	s = "\",\n";
+	send (sock, s, schedwi_strlen (s), 0);
 
-	send (sock, "USR:", 4, 0);
+	s = "\"user\" : \"";
+	send (sock, s, schedwi_strlen (s), 0);
 	send (sock, user_name, schedwi_strlen (user_name), 0);
-	send (sock, "\n", 1, 0);
+	s = "\",\n";
+	send (sock, s, schedwi_strlen (s), 0);
 
-	send (sock, "OUT:", 4, 0);
+	s = "\"stdout\" : \"";
+	send (sock, s, schedwi_strlen (s), 0);
 	send (sock, path_to_cmd, schedwi_strlen (path_to_cmd), 0);
-	send (sock, "/foo.out\n", 9, 0);
+	send (sock, "/foo.out", 8, 0);
+	s = "\",\n";
+	send (sock, s, schedwi_strlen (s), 0);
 
-	send (sock, "ERR:", 4, 0);
+	s = "\"stderr\" : \"";
+	send (sock, s, schedwi_strlen (s), 0);
 	send (sock, path_to_cmd, schedwi_strlen (path_to_cmd), 0);
-	send (sock, "/foo.err\n", 9, 0);
+	send (sock, "/foo.err", 8, 0);
+	s = "\",\n";
+	send (sock, s, schedwi_strlen (s), 0);
 
-	send (sock, "LDE:0\n", 6, 0);
-
-	send (sock, "ARG:2:arg2\n", 11, 0);
-	send (sock, "ARG:1:arg1\n", 11, 0);
-
-	send (sock, "ENV:1:A:1\n", 10, 0); 
-	send (sock, "ENV:4:CMD:$C\n", 13, 0); 
-	send (sock, "ENV:2:B:2\n", 10, 0); 
-	send (sock, "ENV:3:C:foo\n", 12, 0); 
+	
+	s = "\"env\" : [	\"A=1\",\
+				\"B=2\",\
+				\"C=foo\",\
+				\"CMD=$C\"";
+	send (sock, s, schedwi_strlen (s), 0);
 
 	/*
 	 * Load and set the library environment variables that may be required
@@ -298,83 +297,95 @@ start_job (const char *path_to_cmd, const char *user_name)
 	/* Linux, Solaris, HP-UX, Tru64, SGI */
 	env_str = getenv ("LD_LIBRARY_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:10:LD_LIBRARY_PATH:", 23, 0); 
+		s = ", \"LD_LIBRARY_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* Solaris */
 	env_str = getenv ("LD_LIBRARY_PATH_64");
 	if (env_str != NULL) {
-		send (sock, "ENV:11:LD_LIBRARY_PATH_64:", 26, 0); 
+		s = ", \"LD_LIBRARY_PATH_64=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* HP-UX */
 	env_str = getenv ("SHLIB_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:12:SHLIB_PATH:", 18, 0); 
+		s = ", \"SHLIB_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* AIX, OS/390 */
 	env_str = getenv ("LIBPATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:13:LIBPATH:", 15, 0); 
+		s = ", \"LIBPATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* SGI */
 	env_str = getenv ("LD_LIBRARYN32_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:14:LD_LIBRARYN32_PATH:", 26, 0); 
+		s = ", \"LD_LIBRARYN32_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	env_str = getenv ("LD_LIBRARY64_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:15:LD_LIBRARY64_PATH:", 25, 0); 
+		s = ", \"LD_LIBRARY64_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* Windows */
 	env_str = getenv ("PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:16:PATH:", 12, 0); 
+		s = ", \"PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* MacOS X */
 	env_str = getenv ("DYLD_LIBRARY_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:17:DYLD_LIBRARY_PATH:", 25, 0); 
+		s = ", \"DYLD_LIBRARY_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	env_str = getenv ("DYLD_FALLBACK_LIBRARY_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:18:DYLD_FALLBACK_LIBRARY_PATH:", 34, 0); 
+		s = ", \"DYLD_FALLBACK_LIBRARY_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	/* BeOS */
 	env_str = getenv ("LIBRARY_PATH");
 	if (env_str != NULL) {
-		send (sock, "ENV:19:LIBRARY_PATH:", 20, 0); 
+		s = ", \"LIBRARY_PATH=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 	env_str = getenv ("BELIBRARIES");
 	if (env_str != NULL) {
-		send (sock, "ENV:20:BELIBRARIES:", 19, 0); 
+		s = ", \"BELIBRARIES=";
+		send (sock, s, schedwi_strlen (s), 0);
 		send (sock, env_str, schedwi_strlen (env_str), 0); 
-		send (sock, "\n", 1, 0);
+		send (sock, "\"", 1, 0);
 	}
 #endif
 
-	send (sock, "\nbYe", 4, 0);
+	s = " ] } ]";
+	send (sock, s, schedwi_strlen (s), 0);
 
 	/* Read the reply and dump it to stderr */
-	result = 0;
+	result = 1;
 	do{
 		nb_read = recv (sock, buff, 4096, 0);
 		if (nb_read == 0) {
@@ -384,15 +395,17 @@ start_job (const char *path_to_cmd, const char *user_name)
 			closesocket (sock);
 			return 11;
 		}
-		if (result == 0) {
-			result = buff[0];
+		if (result == 1) {
+			if (schedwi_strnstr (buff, "true", nb_read) != NULL) {
+				result = 0;
+			}
 		}
 		write (2, buff, nb_read);
 	} while (1);
 
 	closesocket (sock);
 
-	if (result != '0') {
+	if (result != 0) {
 		return 12;
 	}
 	return 0;
@@ -408,7 +421,7 @@ main (int argc, char **argv)
 	int sock, s, opt, ret;
 	struct sockaddr_in sad;
 	ssize_t nb_read;
-	char buff[4096], *path_to_cmd, *user_name;
+	char buff[4096], *path_to_cmd, *user_name, *str;
 
 	if (argc != 3) {
 		return 1;
@@ -427,7 +440,7 @@ main (int argc, char **argv)
 	}
 
 	/* Set the REUSEADDR flag */
-	opt = -1;
+	opt = 1;
 	setsockopt (sock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof (opt));
 
 	/* Compose the address */
@@ -483,12 +496,13 @@ main (int argc, char **argv)
 	 	 * Write the reply to stdout
 		 */
 		write (1, buff, nb_read);
-		if (schedwi_strnstr (buff, "\nbYe", nb_read) != NULL) {
+		if (schedwi_strnstr (buff, "]", nb_read) != NULL) {
 			break;
 		}
 	} while (1);
 
-	send (s, "0", 1, 0);
+	str = "{ \"success\" : true }";
+	send (s, str, schedwi_strlen (str), 0);
 	closesocket (s);
 	closesocket (sock);
 
